function ComplexPlot

m = 90; n = 152; k = 40; rand('state',1); randn('state',0); % 150,152

% Check if solution already exists
[pathstr,name,ext,versn] = fileparts(mfilename('fullpath'));
filename = [pathstr,filesep,'data',filesep,'ComplexPlot.mat'];

if exist(filename)
   s = load(filename);
   data = s.data;
else
   % Set up problem
   A     = randn(m,n);
   x0    = zeros(n,1);
   p     = randperm(n);
   p     = p(1:k);
   x0(p) = randn(k,1) + sqrt(-1) * randn(k,1);
   b     = A*x0;

   bsplit = [real(b);imag(b)];
   Asplit = [A zeros(m,n); zeros(m,n) A];

   % Set solver options
   opts.spectral    = 1;
   opts.curvilinear = 0;
   opts.maxiter     = 1e4; % 1e5

   % Complex group version
   normx        = norm(x0,1);
   funProjTau   = GroupOneNormProject({n,-2},[],1);
   funProj      = @(x) funProjTau(x,normx);
   funObj       = @(x) objective(Asplit,bsplit,x);
   [x1,g,info1] = spg( funObj, funProj, zeros(2*n,1), opts);

   % Split version
   normx        = norm([imag(x0);real(x0)],1);
   funProjTau   = GroupOneNormProject({2*n,1},[],1);
   funProj      = @(x) funProjTau(x,normx);
   funObj       = @(x) objective(Asplit,bsplit,x);
   [x2,g,info2] = spg( funObj, funProj, zeros(2*n,1), opts);

   % Store results
   data.x0    = x0;
   data.x1    = x1;
   data.x2    = x2;
   data.info1 = info1;
   data.info2 = info2;
   data.n     = n;
   data.m     = m;
   data.k     = k;
   
   save(filename,'data');
end

% Plot complex group results
figure(1); clf; gca; hold on;
h1 = plot(real(data.x0),imag(data.x0),'bo');
h2 = plot(data.x1(1:data.n),data.x1(data.n+1:2*data.n),'r.');
h  = plot([-3,3],[0,0],'k--'); set(h,'Color',[0.7,0.7,0.7]);
h  = plot([0,0],[-3,3],'k--'); set(h,'Color',[0.7,0.7,0.7]);
set(h1,'MarkerSize',9);
set(h1,'LineWidth',2);
set(h2,'MarkerSize',15);
hold off;
set(gca,'Fontsize',16);
legend([h1,h2],'true solution','computed solution','location','northwest');
xlabel('real(x)'); ylabel('imag(x)');
xlim([-3,3]);
ylim([-3,3]);

% Plot split results
figure(2); clf; gca; hold on;
h1 = plot(real(data.x0),imag(data.x0),'bo');
h2 = plot(data.x2(1:data.n),data.x2(data.n+1:2*data.n),'r.');
h  = plot([-3,3],[0,0],'k--'); set(h,'Color',[0.7,0.7,0.7]);
h  = plot([0,0],[-3,3],'k--'); set(h,'Color',[0.7,0.7,0.7]);
set(h1,'MarkerSize',9);
set(h1,'LineWidth',2);
set(h2,'MarkerSize',15);
hold off;
set(gca,'Fontsize',16);
legend([h1,h2],'true solution','computed solution','location','northwest');
xlabel('real(x)'); ylabel('imag(x)');
xlim([-3,3]);
ylim([-3,3]);


function [f,g] = objective(A,b,x)
r = A*x - b;
f = r'*r / 2;
g = (r'*A)';