function QPplot(setname,n)

fs = 16;
lw = 3;

if nargin < 2
   stats = QPbatch(setname);
else
   stats = QPbatch(setname,1:n);
   stats = stats(:,1:n);
end

% Collect runtimes, function values and solvers
T = zeros(size(stats));
F = zeros(size(stats));
S = cell(1,size(stats,1));

for i=1:size(stats,1)
   S{i} = stats{i,1}.solver;
   for j=1:size(stats,2)
      T(i,j) = stats{i,j}.cputime;
      F(i,j) = stats{i,j}.fPenalty;
      N(i,j) = stats{i,j}.feval;
   end
end

figure(1); clf;
idx = [1,5,4,2];
h = boxplot(T(idx,:)','labels',{'BCD','Smooth','PG','SPG'});
set(gca,'YScale','log');
set(gca,'Fontsize',fs);
ylim([1e0,1e3]); ylabel('Runtime (seconds)');


figure(2); clf; gca; hold on;
idx = [1,5,4,2];
h = boxplot(N(idx,:)','labels',{'BCD','Smooth','PG','SPG'});
ylim([1e0,1e4]);
set(gca,'YScale','log');
set(gca,'Fontsize',fs);
set(gca,'YMinorTick','off');
%set(gca,'YGrid','on');
%set(gca,'YMinorGrid','off');
h = plot([0,5],[1e1,1e1],'b--'); set(h,'Color',[0.75,0.75,0.75]);
h = plot([0,5],[1e2,1e2],'b--'); set(h,'Color',[0.75,0.75,0.75]);
h = plot([0,5],[1e3,1e3],'b--'); set(h,'Color',[0.75,0.75,0.75]);
ylabel('Function evaluations');


% $$$ figure(2);
% $$$  idx = [1,2,4,5];
% $$$  M = F(idx,:) - repmat(min(F(idx,:)),length(idx),1) + 1e-12;
% $$$  plot(M');
% $$$  set(gca,'YScale','log')
