function [funObj,tau,groups,weights] = genprobCRF(nNodes)

nInstances = 200;
nStates = 2;
useMex = 1;
edgeProb = 0.5;
tied = 0;
gibbsIters = 10000;
exactInf = 0;

rand('state',0);
randn('state',0);

nFeatures = 10;
ising = 1;

% Generate Data
[y,adjMatrix,X] = UGM_generate(nInstances,nFeatures,nNodes,edgeProb,nStates,ising,tied,0,gibbsIters);
y = double(y);

% Make edges from a full adjacency matrix
adj = fixed_Full(nNodes);
edgeStruct=UGM_makeEdgeStruct(adj);
nEdges = edgeStruct.nEdges;

% Make edge features
Xedge = UGM_makeEdgeFeaturesC(X,int32(edgeStruct.edgeEnds));

% Add bias to each node and edge
X = [ones(nInstances,1,nNodes) X];
Xedge = [ones(nInstances,1,nEdges) Xedge];

% Make Infostruct
infoStruct = UGM_makeInfoStruct(X,Xedge,edgeStruct,nStates,ising,tied,useMex,gibbsIters);

% Initialize weights
[w,v] = UGM_initWeights(infoStruct,'zeros');
wVect = w(infoStruct.wLinInd);
vVect = v(infoStruct.vLinInd);
nNodeVars = length(wVect(:));
nEdgeVars = length(vVect(:));

% Make objective function w/ L2-penalty added to node weights for
% identifiability
lambdaL2 = [1e-4*ones(size(wVect(:)));zeros(size(vVect(:)))];
if exactInf
    gradArgs = {X,Xedge,y,edgeStruct,infoStruct,@UGM_Infer_Exact}
    funObj = @(wv) penalizedL2(wv,@UGM_Loss,lambdaL2,gradArgs{:})
else
    gradArgs = {X,Xedge,y,edgeStruct,infoStruct};
    funObj = @(wv) penalizedL2(wv,@UGM_PseudoLoss,lambdaL2,gradArgs{:});
end

% Form groups
groups = ones(size(v));
for e = 1:nEdges
    groups(:,:,e) = e+1;
end
groups = [ones(nNodeVars,1);groups(infoStruct.vLinInd)];
nGroups = length(unique(groups));

% The weights across groups are equal,
% except we don't penalize the bias elements (group 1)
weights = ones(nGroups,1);
weights(1) = 0;

% Solve unconstrained problem
%[wv] = minFunc(funObj,[zeros(size(wVect(:)));zeros(size(vVect(:)))],[]);

% Set lambda
tau = nNodes;