function  [nodeBel,edgeBel,logZ] = UGM_Infer_IS(nodePot, edgePot, edgeStruct, infoStruct)
% INPUT
% nodePot(node,class)
% edgePot(class,class,edge) where e is referenced by V,E (must be the same
% between feature engine and inference engine)
%
% OUTPUT
% nodeLabel(node)

[nNodes,maxStates] = size(nodePot);
nEdges = size(edgePot,3);
edgeEnds = edgeStruct.edgeEnds;
V = edgeStruct.V;
E = edgeStruct.E;
nStates = infoStruct.nStates;
maxIter = infoStruct.maxIter;

nodeBel = zeros(size(nodePot));
edgeBel = zeros(size(edgePot));
sample = zeros(nNodes,0);
samplePot = zeros(1,0);

% Generate Samples and Compute Weights
y = zeros(nNodes,1);
sample = zeros(nNodes,0);
samplePot = zeros(1,0);
for i = 1:maxIter

    % Generate Random Labeling
    for n = 1:nNodes
       y(n) = ceil(nStates(n)*rand); 
    end

    % Compare sample to unique samples
    % (this would be fast with a hash)

    % Record Sample
    repeated = 0;
    for s = 1:size(sample,2)
        if all(y == sample(:,s))
            %fprintf('Repeated\n');
            repeated = 1;
            break;
        end
    end
    if ~repeated
        %fprintf('New Sample\n');
        sample(:,end+1) = y;

        % Compute Potential of Sample
        pot = UGM_ConfigurationPotential(y,nodePot,edgePot,edgeEnds);
        samplePot(1,end+1) = pot;
        
        % Update nodeBel
        for n = 1:nNodes
            nodeBel(n,y(n)) = nodeBel(n,y(n)) + pot;
        end
        
        % Update edgeBel
        for e = 1:nEdges
            n1 = edgeEnds(e,1);
            n2 = edgeEnds(e,2);
            edgeBel(y(n1),y(n2),e) = edgeBel(y(n1),y(n2),e) + pot;
        end
    end
end

Z = sum(samplePot(:));
nodeBel = nodeBel./Z;
edgeBel = edgeBel./Z;;
logZ = log(Z);