#include <math.h>
#include "mex.h"
#include "UGM_common.h"


void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
   /* Variables */
    int i, n, n1, n2, p, p1, p2, e, e1,e2,s, s1, s2, sInd, edgeInd, edgeInd2,ind,
    nInstances,nNodeFeatures,nEdgeFeatures,nStates,nNodes,nEdges,tied,
    *y, *edgeEnds, *V, *E, sizF[2],
    y_neigh, neigh, y_neigh1, y_neigh2;
    
    double *f, *gw, *gv, *Hw, *Hv, *Hwv, *w, *v, *X, *Xedge, *nodePot, *edgePot,
    *pot, *A, Z, observed, expected, inner;
    
    /* Input */
    gw = mxGetPr(prhs[0]);
    gv = mxGetPr(prhs[1]);
    w = mxGetPr(prhs[2]);
    v = mxGetPr(prhs[3]);
    X = mxGetPr(prhs[4]);
    Xedge = mxGetPr(prhs[5]);
    y = mxGetPr(prhs[6]);
    nodePot = mxGetPr(prhs[7]);
    edgePot = mxGetPr(prhs[8]);
    edgeEnds = mxGetPr(prhs[9]);
    V = mxGetPr(prhs[10]);
    E = mxGetPr(prhs[11]);
    tied = mxGetScalar(prhs[12]);
    Hw = mxGetPr(prhs[13]);
    Hv = mxGetPr(prhs[14]);
    Hwv = mxGetPr(prhs[15]);
    
    /* Compute Sizes */
    nInstances = mxGetDimensions(prhs[4])[0];
    nNodeFeatures = mxGetDimensions(prhs[4])[1];
    nNodes = mxGetDimensions(prhs[4])[2];
    nEdgeFeatures = mxGetDimensions(prhs[5])[1];
    nEdges = mxGetDimensions(prhs[9])[0];
    nStates = 2;
    decrementEdgeEnds(edgeEnds,nEdges);
    for(i = 0; i < nInstances; i++)
    {
        for(n = 0; n < nNodes; n++)
        {
            y[i + nInstances*n]--;
        }
    }
    for(n = 0; n <= nNodes; n++)
    {
        V[n]--;
    }
    for(e = 0; e < nEdges*2; e++)
    {
        E[e]--;
    }
    
    /* Allocate */
    pot = mxCalloc(nStates,sizeof(double));
    A = mxCalloc(nEdgeFeatures,sizeof(double));
    
    /* Output */
    sizF[0] = 1;
    sizF[1] = 1;
    plhs[0] = mxCreateNumericArray(2,sizF,mxDOUBLE_CLASS,mxREAL);
    f = mxGetPr(plhs[0]);
    *f = 0;
    
    for(i = 0; i < nInstances; i++)
    {
        for(n = 0; n < nNodes; n++)
        {
            
            /* Compute NodePot for all States */
            for(s = 0; s < nStates; s++)
            {
                pot[s] = nodePot[n + nNodes*(s + nStates*i)];
            }
            
            /* Add EdgePot for each state with neighbors fixed */
            for(edgeInd = V[n]; edgeInd < V[n+1]; edgeInd++)
            {
                e = E[edgeInd];
                n1 = edgeEnds[e];
                n2 = edgeEnds[e+nEdges];
                if(n == edgeEnds[e])
                {
                    for(s = 0; s < nStates; s++)
                    {
                        pot[s] *= edgePot[s + nStates*(y[i + nInstances*n2] + nStates*(e + nEdges*i))];
                    }
                }
                else
                {
                    for(s = 0; s < nStates; s++)
                    {
                        pot[s] *= edgePot[y[i + nInstances*n1] + nStates*(s + nStates*(e + nEdges*i))];
                    }
                }
            }
            
            /* Update Objective */
            Z = 0;
            for(s = 0; s < nStates; s++)
            {
                Z += pot[s];
            }
            *f = *f - log(pot[y[i + nInstances*n]]) + log(Z);
            
            /* Node Beliefs */
            for(s = 0; s < nStates; s++)
            {
                pot[s] = pot[s]/Z;
            }
            
            /* Update Gradient of Node Weights */
            for(s = 0; s < nStates-1; s++)
            {
                if(s == y[i + nInstances*n])
                {
                    observed = 1;
                }
                else
                {
                    observed = 0;
                }
                expected = pot[s];
                
                for(p = 0; p < nNodeFeatures; p++)
                {
                    if (tied)
                    {
                        gw[p + nNodeFeatures*s] -= (observed - expected)*X[i+nInstances*(p+nNodeFeatures*n)];
                    }
                    else
                    {
                        gw[p + nNodeFeatures*(s + (nStates-1)*n)] -= (observed - expected)*X[i+nInstances*(p+nNodeFeatures*n)];
                    }
                }
            }
            
            
            /* Update Gradient of Edge Weights */
            for(edgeInd = V[n]; edgeInd < V[n+1]; edgeInd++)
            {
                e = E[edgeInd];
                n1 = edgeEnds[e];
                n2 = edgeEnds[e+nEdges];
                
                if(y[i + nInstances*n1] == y[i + nInstances*n2])
                {
                    observed = 1;
                }
                else
                {
                    observed = 0;
                }
                
                if(n == n1)
                {
                    y_neigh = y[i + nInstances*n2];
                }
                else
                {
                    y_neigh = y[i + nInstances*n1];
                }
                
                if(y_neigh < nStates)
                {
                    expected = pot[y_neigh];
                }
                else
                {
                    expected = 0;
                }
                
                for (p = 0; p < nEdgeFeatures; p++)
                {
                    if (tied)
                    {
                        gv[p] -= (observed-expected)*Xedge[i + nInstances*(p + nEdgeFeatures*e)];
                    }
                    else
                    {
                        gv[p+nEdgeFeatures*e] -= (observed-expected)*Xedge[i + nInstances*(p + nEdgeFeatures*e)];
                    }
                }
            }
            
            
            /* Update Hessian of node weights */
            inner = pot[0]*pot[1];
            for(p1 = 0; p1 < nNodeFeatures; p1++)
            {
                for(p2 = 0; p2 < nNodeFeatures; p2++)
                {
                    if (tied)
                    {
                        ind = p1 + nNodeFeatures*p2;
                    }
                    else
                    {
                        ind = p1 + nNodeFeatures*(n + nNodes*(p2 + nNodeFeatures*n));
                    }
                    
                    Hw[ind] += X[i+nInstances*(p1+nNodeFeatures*n)]*inner*X[i+nInstances*(p2+nNodeFeatures*n)];
                }
            }
            
            /* Update Hessian of edge weights wrt node/edge weights */
            if (tied)
            {
                for(p = 0; p < nEdgeFeatures; p++)
                {
                    A[p] = 0;
                }
                for(edgeInd = V[n]; edgeInd < V[n+1]; edgeInd++)
                {
                    e = E[edgeInd];
                    n1 = edgeEnds[e];
                    n2 = edgeEnds[e+nEdges];
                    if(n == n1)
                    {
                        y_neigh = y[i + nInstances*n2];
                    }
                    else
                    {
                        y_neigh = y[i + nInstances*n1];
                    }
                    
                    for(p=0;p<nEdgeFeatures;p++)
                    {
                        if(y_neigh==0)
                        {
                            A[p] += Xedge[i + nInstances*(p + nEdgeFeatures*e)];
                        }
                        else
                        {
                            A[p] -= Xedge[i + nInstances*(p + nEdgeFeatures*e)];
                        }
                    }
                    
                }
                
                for(p1=0;p1 < nEdgeFeatures;p1++)
                {
                    for(p2=0;p2 < nNodeFeatures;p2++)
                    {
                        Hwv[p1 + nEdgeFeatures*p2] += A[p1]*inner*X[i+nInstances*(p2+nNodeFeatures*n)];
                    }
                    for(p2=0;p2 < nEdgeFeatures;p2++)
                    {
                        Hv[p1 + nEdgeFeatures*p2] += A[p1]*inner*A[p2];
                    }
                }
            }
            else /* not tied */
            {
                
                for(edgeInd = V[n]; edgeInd < V[n+1]; edgeInd++)
                {
                    e1 = E[edgeInd];
                    n1 = edgeEnds[e1];
                    n2 = edgeEnds[e1+nEdges];
                    if(n == n1)
                    {
                        y_neigh1 = y[i + nInstances*n2];
                    }
                    else
                    {
                        y_neigh1 = y[i + nInstances*n1];
                    }
                    
                    /* Update edge-node partials */
                    for(p1=0;p1 < nEdgeFeatures;p1++)
                    {
                        for(p2=0;p2 < nNodeFeatures;p2++)
                        {
                            ind = p1 + nEdgeFeatures*(e1 + nEdges*(p2 + nNodeFeatures*n));
                            if(y_neigh1==0)
                            {
                                Hwv[ind] += Xedge[i + nInstances*(p1 + nEdgeFeatures*e1)]*inner*X[i+nInstances*(p2+nNodeFeatures*n)];
                            }
                            else
                            {
                                Hwv[ind] -= Xedge[i + nInstances*(p1 + nEdgeFeatures*e1)]*inner*X[i+nInstances*(p2+nNodeFeatures*n)];
                            }
                        }
                    }
                    
                    /* Update edge-edge partials */
                    for(edgeInd2 = V[n]; edgeInd2 < V[n+1]; edgeInd2++)
                    {
                        e2 = E[edgeInd2];
                        n1 = edgeEnds[e2];
                        n2 = edgeEnds[e2+nEdges];
                        if(n == n1)
                        {
                            y_neigh2 = y[i + nInstances*n2];
                        }
                        else
                        {
                            y_neigh2 = y[i + nInstances*n1];
                        }
                        
                        for(p1=0;p1 < nEdgeFeatures;p1++)
                        {
                            for(p2=0;p2 < nEdgeFeatures;p2++)
                            {
                                ind = p1 + nEdgeFeatures*(e1 + nEdges*(p2 + nEdgeFeatures*e2));
 
                                if (y_neigh1 == y_neigh2)
                                {
                                    Hv[ind] += Xedge[i + nInstances*(p1 + nEdgeFeatures*e1)]*inner*Xedge[i + nInstances*(p2 + nEdgeFeatures*e2)];
                                }
                                else
                                {
                                    Hv[ind] -= Xedge[i + nInstances*(p1 + nEdgeFeatures*e1)]*inner*Xedge[i + nInstances*(p2 + nEdgeFeatures*e2)];
                                }
                            }
                        }
                    }
                    
                    
                }
            }
        }
    }
    
    mxFree(pot);
    mxFree(A);
}