function [samples] = UGM_Sample_MH(nodePot,edgePot,edgeStruct,infoStruct)
% Single Metropolis-Hastings sampler

[nNodes,maxStates] = size(nodePot);
nEdges = size(edgePot,3);
edgeEnds = edgeStruct.edgeEnds;
V = edgeStruct.V;
E = edgeStruct.E;
nStates = infoStruct.nStates;
maxIter = infoStruct.maxIter;

% Initialize
[junk y] = max(nodePot,[],2);
logWeightOld = UGM_LogConfigurationPotential(y,nodePot,edgePot,edgeEnds);

samples = zeros(nNodes,0);
accepted = 0;
for i = 1:maxIter
    
    y_old = y;
    
    % Generate Proposal
    n = mod(i,nNodes)+1;
    transState = [1:y(n)-1 y(n)+1:nStates(n)];
    y(n) = transState(ceil(rand*(nStates(n)-1)));
    
    % Calculate Weight of Proposal
    logWeight = UGM_LogConfigurationPotential(y,nodePot,edgePot,edgeEnds);
    
    % Accept/Reject Proposal
    if rand < exp(logWeight - logWeightOld)
        accepted = accepted+1;
        logWeightOld = logWeight;
    else
        y = y_old;
    end

    % Record Sample
    samples(:,i) = y;

    if mod(i,10) == 0
        fprintf('Accpted = %d out of %d, Acceptance Rate = %.2f\n',accepted,i,accepted/i);
    end
    
end